<?php

class Movie
{
    /**
     * @var string
     */
    private $title;

    /**
     * @var float
     */
    private $rating;

    /**
     * Movie constructor.
     * @param string $title
     * @param float $rating
     */
    public function __construct(string $title, float $rating)
    {
        $this->title = $title;
        $this->rating = $rating;
    }

    /**
     * @return string
     */
    public function title()
    {
        return $this->title;
    }

    /**
     * @return float
     */
    public function rating()
    {
        return $this->rating;
    }
}


class Movies implements \IteratorAggregate
{
    /**
     * @var array
     */
    private $list = [];

    /**
     * add a movie to the movies collection
     *
     * @param Movie $movie
     */
    public function add(Movie $movie): void
    {
        $this->list[] = $movie;
    }

    /**
     * Retrieve an external iterator
     * @link https://php.net/manual/en/iteratoraggregate.getiterator.php
     * @return Traversable An instance of an object implementing <b>Iterator</b> or
     * <b>Traversable</b>
     * @since 5.0.0
     */
    public function getIterator(): iterable
    {
        return new MoviesIterator($this->list);
    }

    /**
     * returns an instance of MoviesIterator with reverse order of movies
     * @return MoviesIterator
     */
    public function getReverseIterator(): iterable
    {
        return new MoviesIterator($this->list, true);
    }

    /**
     * returns a traversable collection of movies with specified rating
     * @param float $rating
     * @return MoviesIterator
     */
    public function getRatedIterator(float $rating): iterable
    {
        $filteredMovies = array_filter($this->list, function (Movie $movie) use ($rating) {
            return $movie->rating() === $rating;
        });

        //reindex filteredMovies from zero
        $filteredMovies = array_values($filteredMovies);

        return new MoviesIterator($filteredMovies);
    }
}


class MoviesIterator implements \Iterator
{
    /**
     * @var array
     */
    private $movieList;

    /**
     * @var int
     */
    private $position = 0;

    /**
     * MoviesIterator constructor.
     * @param array $list
     * @param bool $reverse
     */
    public function __construct(array $list, bool $reverse = false)
    {
        $this->movieList = $reverse ? array_reverse($list) : $list;
    }

    /**
     * Return the current element
     * @link https://php.net/manual/en/iterator.current.php
     * @return mixed Can return any type.
     * @since 5.0.0
     */
    public function current()
    {
        return $this->movieList[$this->position];
    }

    /**
     * Move forward to next element
     * @link https://php.net/manual/en/iterator.next.php
     * @return void Any returned value is ignored.
     * @since 5.0.0
     */
    public function next(): void
    {
        ++$this->position;
    }

    /**
     * Return the key of the current element
     * @link https://php.net/manual/en/iterator.key.php
     * @return mixed scalar on success, or null on failure.
     * @since 5.0.0
     */
    public function key()
    {
        return $this->position;
    }

    /**
     * Checks if current position is valid
     * @link https://php.net/manual/en/iterator.valid.php
     * @return bool The return value will be casted to boolean and then evaluated.
     * Returns true on success or false on failure.
     * @since 5.0.0
     */
    public function valid(): bool
    {
        return isset($this->movieList[$this->position]);
    }

    /**
     * Rewind the Iterator to the first element
     * @link https://php.net/manual/en/iterator.rewind.php
     * @return void Any returned value is ignored.
     * @since 5.0.0
     */
    public function rewind(): void
    {
        $this->position = 0;
    }
}

// movies
$movie1 = new Movie('Movie1', 2.5);
$movie2 = new Movie('Movie2', 7);
$movie3 = new Movie('Movie3', 8);
$movie4 = new Movie('Movie4', 7);
$movie5 = new Movie('Movie5', 9.5);
$movie6 = new Movie('Movie6', 7);

//movie collection
$movieCollection = new Movies();
//add movies to collection
$movieCollection->add($movie1);
$movieCollection->add($movie2);
$movieCollection->add($movie3);
$movieCollection->add($movie4);
$movieCollection->add($movie5);
$movieCollection->add($movie6);

foreach ($movieCollection as $movie) {
    echo '-' . $movie->title() . '<br/>';
}

//reverse order
echo '<br/>' . 'movies in reverse order' . '<br/>';
foreach ($movieCollection->getReverseIterator() as $movie) {
    echo '-' . $movie->title() . '<br/>';
}

//movies with rate 7 only
echo '<br/>' . 'movies with rating =7 only' . '<br/>';
foreach ($movieCollection->getRatedIterator(7) as $movie) {
    echo '-' . $movie->title() . '<br/>';
}


